@@ -102,7 +102,7 @@ def create_instance(region_name='us-east-1', zone_name=None, key_pair=None,
 def create_temp_inst(region=None, zone=None, key_pair=None,
                       security_groups=None, synchronously=False):
     if region and zone:
-        assert zone in region.connect().get_all_zones(), (
+        assert zone in get_region_conn(region.name).get_all_zones(), (
             '{0} doesn\'t belong to {1}'.format(zone, region))
 
     def create_inst_in_zone(zone, key_pair, sec_grps):
@@ -114,7 +114,7 @@ def create_temp_inst(region=None, zone=None, key_pair=None,
     if zone:
         inst = create_inst_in_zone(zone, key_pair, security_groups)
     else:
-        for zone in region.connect().get_all_zones():
+        for zone in get_region_conn(region.name).get_all_zones():
             try:
                 inst = create_inst_in_zone(zone, key_pair, security_groups)
             except BotoServerError as err:
@@ -190,7 +190,8 @@ def mount_volume(vol, mkfs=False):
 
 
 @contextmanager
-def attach_snapshot(snap, key_pair=None, security_groups=None, inst=None):
+def attach_snapshot(snap, key_pair=None, security_groups=None, inst=None,
+                    encr=None):
 
     """Attach `snap` to `inst` or to new temporary instance.
 
@@ -228,7 +229,10 @@ def attach_snapshot(snap, key_pair=None, security_groups=None, inst=None):
         wait_for(inst, 'running')
         try:
             vol, volumes = force_snap_attach(inst, snap)
-            mnt = mount_volume(vol)
+            if encr:
+                mnt = None
+            else:
+                mnt = mount_volume(vol)
             yield vol, mnt
         except BaseException as err:
             logger.exception(str(err))
@@ -236,10 +240,11 @@ def attach_snapshot(snap, key_pair=None, security_groups=None, inst=None):
             key_filename = config.get(inst.region.name, 'KEY_FILENAME')
             with settings(host_string=inst.public_dns_name,
                           key_filename=key_filename):
-                try:
-                    wait_for_sudo('umount {0}'.format(mnt))
-                except:
-                    pass
+                if not encr:
+                    try:
+                        wait_for_sudo('umount {0}'.format(mnt))
+                    except:
+                        pass
             for vol in volumes:
                 if vol.status != 'available':
                     vol.detach(force=True)
@@ -376,7 +381,7 @@ def make_encrypted_ubuntu(host_string, key_filename, user,
             sudo('mount -o loop,ro "{work}/{image}" "{work}/ubuntu"'
                  .format(image=image, work=work))
             logger.info('Creating separate boot volume.....')
-            sudo('echo -e "0 1024 83 *\n;\n" | /sbin/sfdisk -uM {dev}'
+            sudo('echo -e "0 512 83 *\n;\n" | /sbin/sfdisk -uM {dev}'
                  .format(dev=dev))
             logger.info('Formatting boot volume.....')
             sudo('/sbin/mkfs -t ext3 -L "{bootlabel}" "{dev}1"'
@@ -497,7 +502,7 @@ def make_encrypted_ubuntu(host_string, key_filename, user,
             sudo('chroot "{work}/root" <<- EOT\n'
                  'apt-get -y install cryptsetup\n'
                  'apt-get -y clean\n'
-                 #'update-initramfs -uk all\n'
+                 'update-initramfs -uk all\n'
                  'mv /etc/resolv.conf.old /etc/resolv.conf\n'
                  'umount /dev/pts\n'
                  'umount /proc\n'
@@ -724,6 +729,9 @@ def create_ami(region=None, snap_id=None, force=None, root_dev='/dev/sda1',
         instance_type = get_descr_attr(snap, 'Type') or default_type
         new_instance = launch_instance_from_ami(region, image.id,
                                                 inst_type=instance_type)
+    if dev:
+        logger.info('\nTo unlock go to:\n   https://{0}\n'
+                        .format(new_instance.public_dns_name))
     return image, new_instance
 
 
@@ -734,11 +742,6 @@ def create_encrypted_instance(region_name, release='lucid', volume_size='8',
     """
     Creates ubuntu instance with luks-encryted root volume.
 
-    .. note:: Snapshot replication to backup region with `rsync_region`
-              and `rsync_snapshot` doesn't function for encrypted
-              volumes. Due to this reason encrypted instance could be
-              restored only within its region.
-
     region_name
         Region where you want to create instance;
     release
