@@ -277,7 +277,7 @@ def get_snap_time(snap):
 
 
 def get_inst_by_id(region, instance_id):
-    res = region.connect().get_all_instances([instance_id, ])
+    res = get_region_conn(region.name).get_all_instances([instance_id, ])
     assert len(res) == 1, (
         'Returned more than 1 {0} for instance_id {1}'.format(res,
                                                       instance_id))
@@ -317,7 +317,8 @@ def update_volumes_tags(filters=None):
                     :func:`django_fabfile.utils.get_all_instances`.
     """
     for region in regions():
-        reservations = region.connect().get_all_instances(filters=filters)
+        reservations = get_region_conn(region.name).get_all_instances(
+            filters=filters)
         for res in reservations:
             inst = res.instances[0]
             for bdm in inst.block_device_mapping.keys():
